@@ -1,5 +1,5 @@
 # prime/prime.rb
-# $Id: prime.rb,v 1.12 2004/06/24 07:28:44 komatsu Exp $
+# $Id: prime.rb,v 1.13 2004/08/29 12:49:28 komatsu Exp $
 #
 # Copyright (C) 2002, 2003, 2004 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -121,7 +121,7 @@ class Prime
   end
 
   def preedit_convert_input (pattern)
-    return PrimeTypeConv::convert(pattern).join()
+    return PrimeTypeConv::convert(pattern).join("\t")
   end
 
   def set_context (context)
@@ -179,7 +179,7 @@ class Prime
 
   def lookup_direct (string)
     results_expansion = lookup_expansion(string)
-    results = PrimeWordList::merge_with_label(@context, results_expansion)
+    results = PrimeWordList::merge_by_literal(@context, results_expansion)
     return results[0,3]
   end
 
@@ -190,7 +190,7 @@ class Prime
     words_japanese  = search_japanese(string)
     words_overall   = search_overall(string)
 
-    results = PrimeWordList::merge_with_label(@context,
+    results = PrimeWordList::merge_by_literal(@context,
                                               # results_prefix[0, 3],
                                               results_expansion,
                                               results_mixed,
@@ -520,6 +520,35 @@ class PrimeWordList < Array
     return merged
   end
 
+  def PrimeWordList::merge_by_literal (context, *words)
+    ## merge should be sorted by inter-engine score.
+    ## FIXME: Isn't it necessary to sort here?
+    ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
+    result = words.flatten.compact.sort {|word1, word2| 
+      (word1.score != word2.score) ? (word2.score <=> word1.score) :
+                                     (word2.pron.length <=> word1.pron.length)
+    }
+
+    mark = {}
+    merged = PrimeWordList.new
+    result.each {|word|
+      if word.literal != "" then
+        if mark[word.literal] then
+          word0 = mark[word.literal]
+          ## FIXME: Make this merging more intelligence.
+          ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
+          word0.category   = (word0.category   or word.category)
+          word0.annotation = (word0.annotation or word.annotation)
+        else
+          mark[word.literal] = word
+          word.prefix = Prime::get_prefix(context, word.literal)
+          merged.push(word)
+        end
+      end
+    }
+    return merged
+  end
+
   def to_text
     texts = self.map {|word|
       word.to_text
